@@ -1,11 +1,25 @@
 """Module to help draw plotly shapes, drawing and annotations"""
 
-from plotly.subplots import make_subplots
-import plotly.graph_objects as go
+# Standard Library Imports
+from math import radians
+
+# Third Party Imports
 import numpy as np
 from sympy import lambdify, sympify, sin, cos, oo
 from sympy.abc import x
-from math import radians
+from plotly.subplots import make_subplots
+import plotly.graph_objects as go
+
+# Local Application Imports
+from indeterminatebeam.loading import(
+    PointLoad,
+    PointLoadH,
+    PointLoadV,
+    PointTorque,
+    UDL,
+    TrapezoidalLoad,
+    DistributedLoad,
+)
 
 
 def draw_line(fig, angle, x_sup, length=-20, xoffset=0, yoffset=0,
@@ -143,7 +157,8 @@ def draw_arrowhead(fig, angle, x_sup, length=5, xoffset=0, yoffset=0,
 
 
 def draw_arrow(fig, angle, force, x_sup, xoffset=0, yoffset=0, color='red',
-               line_width=2, arrowhead=5, arrowlength=30, show_values=True,row=None, col=None):
+               line_width=2, arrowhead=5, arrowlength=40, show_values=True,
+               row=None, col=None):
     """Draw an anchored arrow on a plotly figure.
 
     Parameters
@@ -171,6 +186,8 @@ def draw_arrow(fig, angle, force, x_sup, xoffset=0, yoffset=0, color='red',
         Size of the arrowhead lines, by default 5
     arrowlength: int, optional
         length of the arrow line, by default 30
+    show_values: bool,optional
+        If true annotates numerical force value next to arrow, by default True.
     row : int or None,
         Row of subplot to draw line on. If None specified assumes a full plot,
         by default None.
@@ -190,8 +207,10 @@ def draw_arrow(fig, angle, force, x_sup, xoffset=0, yoffset=0, color='red',
     # Factor to switch arrow direction based on force sign
     if force > 0:
         d = 1
-    else:
+    elif force < 0:
         d = -1
+    else:
+        return fig
 
     # Draw arrowhead for force
     fig = draw_arrowhead(
@@ -222,15 +241,21 @@ def draw_arrow(fig, angle, force, x_sup, xoffset=0, yoffset=0, color='red',
         # determine start and end of arrow
         x0 = xoffset + x_sup
         y0 = yoffset
-        x1 = (x0 + int(sympify(-arrowlength* d * cos(radians(angle))).evalf(2)))*1.1
-        y1 = (y0 + int(sympify(-arrowlength * d * sin(radians(angle))).evalf(2)))*1.3
-        
+        x1 = (
+            x0
+            + int(sympify(-arrowlength * d * cos(radians(angle))).evalf(2))
+            ) * 1.1
+        y1 = (
+            y0
+            + int(sympify(-arrowlength * d * sin(radians(angle))).evalf(2))
+            ) * 1.3
+
         # make so text doesnt intersect x axis
-        if abs(y1)<5:
+        if abs(y1) < 5:
             if y1 >= 0:
-                y1=10
+                y1 = 10
             else:
-                y1=-10
+                y1 = -10
 
         annotation = dict(
             xref="x", yref="y",
@@ -241,7 +266,7 @@ def draw_arrow(fig, angle, force, x_sup, xoffset=0, yoffset=0, color='red',
             text=force,
             font_color=color,
             showarrow=False,
-            )
+        )
 
         # Append shape to plot or subplot
         if row and col:
@@ -360,8 +385,14 @@ def draw_support_rectangle(fig, x_sup, orientation="up", row=None, col=None):
     return fig
 
 
-def draw_moment(fig, moment, x_sup, color='magenta', show_values=True, row=None,
-                col=None):
+def draw_moment(
+        fig,
+        moment,
+        x_sup,
+        color='magenta',
+        show_values=True,
+        row=None,
+        col=None):
     """Draw a moment (torque) shape (circular arrow) on a plotly figure.
 
     Parameters
@@ -373,7 +404,9 @@ def draw_moment(fig, moment, x_sup, color='magenta', show_values=True, row=None,
     direction : 'clockwise' or 'anti-clockwise', optional
         direction for circular arrow to point in, default 'clockwise
     color : str, optional
-        color of circular arrow, default 'red'
+        color of circular arrow, default 'magenta'
+    show_values: bool,optional
+        If true annotates numerical force value next to arrow, by default True.
     row : int or None,
         Row of subplot to draw line on. If None specified assumes a full plot,
         by default None.
@@ -422,7 +455,7 @@ def draw_moment(fig, moment, x_sup, color='magenta', show_values=True, row=None,
             text=moment,
             font_color=color,
             showarrow=False,
-            )
+        )
 
         # Append shape to plot or subplot
         if row and col:
@@ -442,8 +475,6 @@ def draw_force(fig, load, row=None, col=None):
         plotly figure to append force representation to.
     load : instance of a load class
         force to be represented on figure
-    color : str, optional
-        color of force representation, default 'red'
     row : int or None,
         Row of subplot to draw line on. If None specified assumes a full plot,
         by default None.
@@ -457,13 +488,9 @@ def draw_force(fig, load, row=None, col=None):
         Returns the plotly figure passed into function with the force
         representation appended to it.
     """
-    # Get namedtuple name to check the type of load.  (Is used as an
-    # alternative to isinstance method in order to avoid importing the
-    # load classes to avoid circular dependency.)
-    load_type = str(load).split('(')[0]
 
-    if load_type == 'PointTorque':
-        moment, x_sup = load
+    if isinstance(load, PointTorque):
+        moment, x_sup = load.force, load.position
         fig = draw_moment(
             fig,
             moment,
@@ -471,9 +498,8 @@ def draw_force(fig, load, row=None, col=None):
             row=row,
             col=col)
 
-    elif load_type == 'PointLoadH':
-        force, x_sup = load
-        angle = 0
+    elif isinstance(load, PointLoad):
+        force, x_sup, angle = load.force, load.position, load.angle
 
         fig = draw_arrow(
             fig,
@@ -483,44 +509,45 @@ def draw_force(fig, load, row=None, col=None):
             row=row,
             col=col)
 
-    elif load_type == 'PointLoadV':
-        force, x_sup = load
-        angle = 90
+    elif isinstance(load, (DistributedLoad, UDL, TrapezoidalLoad)):
+        angle = load.angle
+        if angle % 90 == 0:
+            # vertical or horizontal
+            color = 'mediumpurple'
+        if angle % 180 == 0:
+            # horizontal only
+            color = 'maroon'
+        elif angle % 90 != 0:
+            color = 'green'
 
-        fig = draw_arrow(
-            fig,
-            angle,
-            force,
-            x_sup,
-            row=row,
-            col=col)
+        if sin(angle) >= 0:
+            angle_factor = -1
+        else:
+            angle_factor = 1
 
-    elif load_type == 'PointLoad':
-        force, x_sup, angle = load
+        if isinstance(load, DistributedLoad):
+            name = 'Distributed<br>Load'
+            x0, x1 = load.span
+            expr = load.expr
+            # numpy array for x positions closely spaced (allow for graphing)
+            x_vec = np.linspace(x0, x1, int(min((x1 - x0) * 100 + 1, 1e3)))
+            y_lam = lambdify(x, expr, 'numpy')
+            y_vec = np.array([round(float(y_lam(t)), 3) for t in x_vec])
 
-        fig = draw_arrow(
-            fig,
-            angle,
-            force,
-            x_sup,
-            row=row,
-            col=col)
-
-    elif load_type == 'DistributedLoadV':
-        # need to know sign for each side.
-        # draw each function normalised to 1. ie the max is always 1.
-        # evaluate force at left and force at right to plot
-        expr, interval = load
-        x0, x1 = interval
-        expr = sympify(expr)
-        # numpy array for x positions closely spaced (allow for graphing)
-        x_vec = np.linspace(x0, x1, int(min((x1 - x0) * 100 + 1, 1e4)))
-        y_lam = lambdify(x, expr, "numpy")
-        y_vec = np.array([y_lam(t) for t in x_vec])
+        elif isinstance(load, UDL):
+            name = 'UDL'
+            x_vec = np.array(load.span)
+            y_vec = np.array([load.force, load.force])
+        else:
+            name = 'Trapezoidal<br>Load'
+            x_vec = np.array(load.span)
+            y_vec = np.array(load.force)
 
         largest = abs(max(y_vec, key=abs))
-        # normalise  , use -1 to flip direction so matches arrow direction
-        y_vec = (-1) * y_vec / largest
+
+        # draw each function normalised to 1. ie the max is always 1.
+        # largest accounts for magnitude, angle factor rectifies polarity.
+        y_vec = (angle_factor) * y_vec / largest
 
         # Create trace object for graph of distributed force
         trace = go.Scatter(
@@ -528,10 +555,10 @@ def draw_force(fig, load, row=None, col=None):
             y=y_vec.tolist(),
             mode='lines',
             line=dict(
-                color='darkgreen',
+                color=color,
                 width=1),
             fill='tozeroy',
-            name='DistributedLoadV',
+            name=name,
             hovertemplate="",
             hoverinfo="skip")
 
@@ -543,28 +570,17 @@ def draw_force(fig, load, row=None, col=None):
 
         # draw arrow for left force and right force (if larger than 2% of
         # max load)
-        angle = 90
-        if abs(y_vec[0]) > 0.02:
-            fig = draw_arrow(
-                fig,
-                angle,
-                -y_vec[0] * largest,
-                x_vec[0],
-                color='green',
-                arrowlength=30*abs(y_vec[0]),
-                row=row, 
-                col=col)
-
-        if abs(y_vec[-1]) > 0.02:
-            fig = draw_arrow(
-                fig,
-                angle,
-                -y_vec[-1]* largest,
-                x_vec[-1],
-                color='green',
-                arrowlength=30*abs(y_vec[-1]),
-                row=row,
-                col=col)
+        for a in [0, -1]:
+            if abs(y_vec[a]) > 0.02:
+                fig = draw_arrow(
+                    fig,
+                    angle,
+                    angle_factor * y_vec[a] * largest,
+                    x_vec[a],
+                    color=color,
+                    arrowlength=30 * abs(y_vec[a]),
+                    row=row,
+                    col=col)
 
     return fig
 
@@ -591,36 +607,22 @@ def draw_load_hoverlabel(fig, load, row=None, col=None):
     plotly figure
         plotly figure to append hoverlabel representation to.
     """
-
-    # Get namedtuple name to check the type of load.  (Is used as an
-    # alternative to isinstance method in order to avoid importing the
-    # load classes to avoid circular dependency.)
-    load_type = str(load).split('(')[0]
-
     y_sup = 0
 
-    if load_type in ['PointLoad', 'PointLoadH', 'PointLoadV', 'PointTorque']:
-        x_sup = load[1]
+    if isinstance(load, (PointLoad, PointTorque)):
+        x_sup = load.position
         color = 'red'
-        meta = [load[0], load[1]]  # load, position
+        meta = [load.force, load.position]  # load, position
 
-        if load_type == 'PointTorque':
+        if isinstance(load, PointTorque):
             color = 'magenta'
             hovertemplate = 'x: %{meta[1]} m<br>Moment: %{meta[0]} kN.m'
             name = 'Point<br>Torque'
-        elif load_type == 'PointLoad':
-            meta.append(load[2])
-            hovertemplate = 'x: %{meta[1]} m<br>Force: %{meta[0]} \
-                kN<br>Angle: %{meta[2]} deg'
+        elif isinstance(load, PointLoad):
+            meta.append(load.angle)
+            hovertemplate = 'x: %{meta[1]} m<br>Force: %{meta[0]} kN\
+                <br>Angle: %{meta[2]} deg'
             name = 'Point<br>Load'
-        elif load_type == 'PointLoadV':
-            hovertemplate = 'x: %{meta[1]} m<br>Force: %{meta[0]} \
-                kN<br>Direction: Vertical'
-            name = 'Point<br>LoadV'
-        else:
-            hovertemplate = 'x: %{meta[1]} m<br>Force: %{meta[0]} \
-                kN<br>Angle: Horizontal'
-            name = 'Point<br>LoadH'
 
         # Define hoverlabel as a marker with 0 opacity and a hovertemplate that
         # relies on meta data field
@@ -643,48 +645,58 @@ def draw_load_hoverlabel(fig, load, row=None, col=None):
 
     # Else is distributed load type, hoverlabel needed for arrow at each side
     # of function
-    else:
-        expr, interval = load
-        x0, x1 = interval
-        expr = sympify(expr)
-        # numpy array for x positions closely spaced (allow for graphing)
-        
-        y_lam = lambdify(x, expr, "numpy")
+    elif isinstance(load, (DistributedLoad, UDL, TrapezoidalLoad)):
+        if load.angle % 90 == 0:
+            # vertical or horizontal
+            color = 'mediumpurple'
+        if load.angle % 180 == 0:
+            # horizontal only
+            color = 'maroon'
+        elif load.angle % 90 != 0:
+            color = 'green'
 
+        x0, x1 = load.span
+        angle = load.angle
         name = 'Distributed<br>Load'
-        color = 'green'
-        y_sup = 1
 
-        meta = [x0, x1, y_lam(x0), y_lam(x1)]
+        if isinstance(load, DistributedLoad):
+            expr = load.expr
+            meta = [
+                (x0, round(float(expr.subs(x, x0)), 3), angle),
+                (x1, round(float(expr.subs(x, x1)), 3), angle)
+            ]
 
-        trace_1 = go.Scatter(
-            x=[x0], y=[0],
-            showlegend=False, mode="markers",
-            name=name,
-            meta=meta,
-            marker=dict(symbol="triangle-up", size=10, color=color),
-            hovertemplate='x: %{meta[0]} m<br>Force: %{meta[2]} kN/m',
-            hoverinfo="skip",
-            opacity=0
-        )
-
-        trace_2 = go.Scatter(
-            x=[x1], y=[0],
-            showlegend=False, mode="markers",
-            name=name,
-            meta=meta,
-            marker=dict(symbol="triangle-up", size=10, color=color),
-            hovertemplate='x: %{meta[1]} m<br>Force: %{meta[3]} kN/m',
-            hoverinfo="skip",
-            opacity=0
-        )
-
-        if row and col:
-            fig.add_trace(trace_1, row=row, col=col)
-            fig.add_trace(trace_2, row=row, col=col)
+        elif isinstance(load, UDL):
+            meta = [
+                (x0, load.force, angle),
+                (x1, load.force, angle)
+            ]
         else:
-            fig.add_trace(trace_1)
-            fig.add_trace(trace_2)
+            meta = [
+                (x0, load.force[0], angle),
+                (x1, load.force[1], angle)
+            ]
+        
+        
+        hovertemplate = 'x: %{meta[0]} m<br>Force: %{meta[1]} kN/m<br>\
+            Angle: %{meta[2]} deg'
+
+        for x_, y_, a_ in meta:
+            trace = go.Scatter(
+                x=[x_], y=[0],
+                showlegend=False, mode="markers",
+                name=name,
+                meta=[x_, y_, a_],
+                marker=dict(symbol="triangle-up", size=10, color=color),
+                hovertemplate=hovertemplate,
+                hoverinfo="skip",
+                opacity=0
+            )
+
+            if row and col:
+                fig.add_trace(trace, row=row, col=col)
+            else:
+                fig.add_trace(trace)
 
     return fig
 
@@ -717,14 +729,14 @@ def draw_reaction_hoverlabel(fig, reactions, x_sup, row=None, col=None):
     x_, y_, m_ = reactions
 
     # Write hovertemplate depending on support restraints
-    hovertemplate = "Reactions"
+    hovertemplate = "Reactions<br>x coord: %{x} m"
     if x_:
         hovertemplate += "<br>x: %{meta[0]} kN"
     if y_:
         hovertemplate += "<br>y: %{meta[1]} kN"
     if m_:
         hovertemplate += "<br>m: %{meta[2]} kN.m"
-    
+
     # Create scatter object with opacity 0 for hovertemplate
     trace = go.Scatter(
         x=[x_sup], y=[0],
@@ -736,7 +748,7 @@ def draw_reaction_hoverlabel(fig, reactions, x_sup, row=None, col=None):
         hoverinfo="skip",
         opacity=0
     )
-    
+
     # Add to plot or subplot
     if row and col:
         fig.add_trace(trace, row=row, col=col)
@@ -794,7 +806,7 @@ def draw_support_hoverlabel(fig, support, kx=0, ky=0, row=None, col=None):
             hovertemplate += "<br>kx: %{meta[0]} kN/mm"
         if ky:
             hovertemplate += "<br>ky: %{meta[1]} kN/mm"
-    
+
     # Support
     else:
         name = "Support"
@@ -857,7 +869,7 @@ def draw_support_rollers(fig, x_sup, orientation='up', offset=1, row=None,
     radius = 1
     if orientation in ['up', 'right']:
 
-        # shifting the position from x_sup to make more aesthetic for 
+        # shifting the position from x_sup to make more aesthetic for
         # triangle or rectangle.
         # A triangle is wider so an offset of 1 is used.
         # A rectangle uses an offset shorter, currently 0.6 is used.
@@ -893,7 +905,14 @@ def draw_support_rollers(fig, x_sup, orientation='up', offset=1, row=None,
     return fig
 
 
-def draw_support_spring(fig, support, orientation="up",color='orange',show_values=True, row=None, col=None):
+def draw_support_spring(
+        fig,
+        support,
+        orientation="up",
+        color='orange',
+        show_values=True,
+        row=None,
+        col=None):
     """Draw an anchored spring shape on a plotly figure.
 
     Parameters
@@ -904,6 +923,10 @@ def draw_support_spring(fig, support, orientation="up",color='orange',show_value
         support to be represented on figure
     orientation : 'up' or 'right, optional
         direction that the arrow faces, by default "up"
+    color : str, optional
+        color of spring, by default 'orange'.
+    show_values: bool,optional
+        If true annotates numerical force value next to arrow, by default True.
     row : int or None,
         Row of subplot to draw line on. If None specified assumes a full plot,
         by default None.
@@ -965,18 +988,18 @@ def draw_support_spring(fig, support, orientation="up",color='orange',show_value
             x0, y0 = x1, y1
 
         if show_values:
-            y0 = max(y0,7)
+            y0 = max(y0, 7)
 
             annotation = dict(
                 xref="x", yref="y",
                 x=x_sup,
                 y=0,
-                yshift=y0*1.5,
-                xshift=x0*2,
+                yshift=y0 * 1.5,
+                xshift=x0 * 2,
                 text=stiffness,
                 font_color=color,
                 showarrow=False,
-                )
+            )
 
             # Append shape to plot or subplot
             if row and col:
@@ -984,7 +1007,6 @@ def draw_support_spring(fig, support, orientation="up",color='orange',show_value
             else:
                 fig.add_annotation(annotation)
 
-
     return fig
 
 
@@ -1031,7 +1053,7 @@ def draw_support(fig, support, row=None, col=None):
         if fixed == [0, 0, 1]:
             fig = draw_moment(
                 fig,
-                moment = 1,
+                moment=1,
                 x_sup=support._position,
                 color='blue',
                 show_values=False,
